#!/usr/bin/env python
# -*- coding: utf-8 -*-

from yowsup.layers.auth.protocolentities import SuccessProtocolEntity, FailureProtocolEntity
from yowsup.layers.protocol_acks.protocolentities.test_ack_incoming import entity as ack_incoming_entity
from yowsup.layers.protocol_acks.protocolentities import IncomingAckProtocolEntity
from yowsup.layers.protocol_messages.protocolentities import TextMessageProtocolEntity
from yowsup.layers.protocol_receipts.protocolentities import IncomingReceiptProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_image \
    import DownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_image \
    import ImageDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_audio import \
    AudioDownloadableMediaMessageProtocolEntity
import time


def success_protocol_entity():
    attributes = {
        "status": "active",
        "kind": "free",
        "creation": "1234",
        "expiration": "1446578937",
        "props": "2",
        "t": "1415470561"
    }
    return SuccessProtocolEntity(**attributes)

def failure_protocol_entity():
    attributes = {
        "reason": "Auth incorrect"
    }
    return FailureProtocolEntity(**attributes)


def ack_incoming_protocol_entity(message=None):
    if not message:
        return ack_incoming_entity
    attributes = {
        "_id": message.getId(),
        "_class": message.getType(),
        "_from": message.getTo(),
        "timestamp": int(time.time())                  
    }
    return IncomingAckProtocolEntity(**attributes)

def text_message_protocol_entity():
    attributes = {
        "body": "received message",
        "_from": "bbb@s.whatsapp.net"
    }
    return TextMessageProtocolEntity(**attributes)

def receipt_incoming_protocol_entity():
    attributes = {
        "_id": "123",
        "_from": "sender",
        "timestamp": int(time.time())
    }
    return IncomingReceiptProtocolEntity(**attributes)

def image_downloadable_media_message_protocol_entity(url="url", to="341234567", ip="ip", caption="image caption"):
    attributes = {
        "mimeType": DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_IMAGE,
        "fileHash": "filehash",
        "url": url,
        "ip": ip,
        "size": "1234",
        "fileName": "image.jpg",
        "encoding": "encoding",
        "width": "123",
        "height": "123",
        "caption": caption,
        "to": to
    }
    return ImageDownloadableMediaMessageProtocolEntity(**attributes)

def audio_downloadable_media_message_protocol_entity(url="url", to="341234567", ip="ip", caption="image caption"):
    attributes = {
        "mimeType": DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_AUDIO,
        "fileHash": "filehash",
        "url": url,
        "ip": ip,
        "size": "1234",
        "fileName": "audio.mp3",
        "abitrate": "arbitrate",
        "acodec": "acodec",
        "asampfreq": "asampfreq",
        "duration": "10",
        "origin": "origin",
        "seconds": "100",        
        "encoding": "encoding",
        "to": to
    }
    return AudioDownloadableMediaMessageProtocolEntity(**attributes)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from yowsup_celery import tasks
from celery import current_app
try:
    from unittest import mock
except ImportError:
    import mock  # noqa

class TestTasks(unittest.TestCase):
    
    def setUp(self):
        current_app.conf.CELERY_ALWAYS_EAGER = True

    def tearDown(self):
        pass
    
    def test_only_one_listen(self):
        with mock.patch('yowsup_celery.tasks.YowsupTask.stack', new_callable=mock.PropertyMock) as mock_stack:
            mock_stack.return_value = mock.MagicMock(listening=True)
            mock_stack.return_value.stack = mock.MagicMock()
            tasks.listen()
            self.assertEqual(0, mock_stack.asynloop.call_count)
            
    def test_listen(self):
        with mock.patch('yowsup_celery.tasks.YowsupTask.stack', new_callable=mock.PropertyMock) as mock_stack:
            mock_stack.return_value = mock.MagicMock(listening=False)
            mock_stack.return_value.stack = mock.MagicMock()
            tasks.listen()
            mock_stack.return_value.asynloop.assert_called_once_with()
                        
    def test_listen_required(self):
        with mock.patch('yowsup_celery.tasks.YowsupTask.stack', new_callable=mock.PropertyMock) as mock_stack, \
                mock.patch('yowsup_celery.tasks.YowsupTask.facade', new_callable=mock.PropertyMock) as mock_facade, \
                mock.patch('celery.app.task.Context', new_callable=mock.MagicMock) as mock_request, \
                mock.patch('yowsup_celery.tasks.connect.apply_async', new_callable=tasks.listen), \
                mock.patch('yowsup_celery.tasks.connect.retry', new_callable=tasks.connect):
            mock_request.return_value = mock.MagicMock(delivery_info={'routing_key': 'any_queue'})
            mock_stack.return_value = mock.MagicMock(listening=False)
            mock_stack.return_value.stack = mock.MagicMock()
            tasks.connect()
            mock_stack.return_value.asynloop.assert_called_once_with()
            mock_facade.return_value.connect.assert_called_once_with()
            
    def test_listen_required_not_needed(self):
        with mock.patch('yowsup_celery.tasks.YowsupTask.stack', new_callable=mock.PropertyMock) as mock_stack, \
                mock.patch('yowsup_celery.tasks.YowsupTask.facade', new_callable=mock.PropertyMock) as mock_facade:
            mock_stack.return_value = mock.MagicMock(listening=True)
            mock_stack.return_value.stack = mock.MagicMock()
            tasks.connect()
            self.assertEqual(0, mock_stack.asynloop.call_count)
            mock_facade.return_value.connect.assert_called_once_with()
            
if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



# -*- coding: utf-8 -*-
from yowsup.layers.interface import YowInterfaceLayer, ProtocolEntityCallback
from yowsup.layers import EventCallback
from yowsup.layers.protocol_messages.protocolentities import \
    TextMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities import RequestUploadIqProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_image import \
    ImageDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable_audio import \
    AudioDownloadableMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_location import LocationMediaMessageProtocolEntity
from yowsup.layers.protocol_media.protocolentities.message_media_vcard import VCardMediaMessageProtocolEntity
import logging
from yowsup.layers.network import YowNetworkLayer
from yowsup.layers import YowLayerEvent
from functools import wraps
import sys
import os
from yowsup_celery.layer_interface import CeleryLayerInterface
from yowsup_celery.exceptions import ConnectionError
from yowsup.layers.protocol_media.mediauploader import MediaUploader

logger = logging.getLogger(__name__)

def connection_required(f):
    @wraps(f)
    def decorated_function(self, *args, **kwargs):
        if not self.connected:
            raise ConnectionError("%s needs to be connected" % f.__name__)
        return f(self, *args, **kwargs)
    return decorated_function

class CeleryLayer(YowInterfaceLayer):    
    """
    Layer to be on the top of the Yowsup Stack. 
    :ivar bool connected: connected or not connected to whatsapp
    :ivar YowLayerInterface interface: layer interface
    """

    def __init__(self):
        super(CeleryLayer, self).__init__()
        self.connected = False
        self.interface = CeleryLayerInterface(self)

    def normalize_jid(self, number):
        if '@' in number:
            return number
        elif "-" in number:
            return "%s@g.us" % number

        return "%s@s.whatsapp.net" % number
    
    def do_send_image(self, file_path, url, to, ip=None, caption=None):
        entity = ImageDownloadableMediaMessageProtocolEntity.fromFilePath(file_path, url, ip, to, caption=caption)
        self.toLower(entity)

    def do_send_audio(self, file_path, url, to, ip=None, caption=None):
        entity = AudioDownloadableMediaMessageProtocolEntity.fromFilePath(file_path, url, ip, to)
        self.toLower(entity)
        
    @ProtocolEntityCallback("success")
    def on_success(self, success_protocol_entity):
        """
        Callback when there is a successful connection to whatsapp server
        """
        logger.info("Logged in")
        self.connected = True
            
    @ProtocolEntityCallback("failure")
    def on_failure(self, entity):
        """
        Callback function when there is a failure in a connection to whatsapp 
        server
        """
        logger.error("Login failed, reason: %s" % entity.getReason())
        self.connected = False
        
    @ProtocolEntityCallback("ack")
    @connection_required
    def on_ack(self, entity):
        """
        Callback function when receiving an ack for a sent message from 
        whatsapp
        """
        logger.info("Ack id %s received" % entity.getId())
   
    @ProtocolEntityCallback("message")
    @connection_required
    def on_message(self, message_protocol_entity):
        """
        Callback function when receiving message from whatsapp server
        """
        logger.info("Message id %s received" % message_protocol_entity.getId())
        # answer with receipt
        self.toLower(message_protocol_entity.ack())
            
    @ProtocolEntityCallback("receipt")
    @connection_required
    def on_receipt(self, receipt_protocol_entity):
        """
        Callback function when receiving receipt message from whatsapp
        """
        self.toLower(receipt_protocol_entity.ack())
        
    @EventCallback(YowNetworkLayer.EVENT_STATE_DISCONNECTED)
    @connection_required
    def on_disconnected(self, yowLayerEvent):
        """
        Callback function when receiving a disconnection event
        """
        logger.info("On disconnected")
        self.connected = False
        
    def on_request_upload_result(self, jid, path, result_request_upload_iq_protocol_entity, 
                                 request_upload_iq_protocol_entity, caption=None):

        if request_upload_iq_protocol_entity.mediaType == RequestUploadIqProtocolEntity.MEDIA_TYPE_AUDIO:
            do_send_fn = self.do_send_audio
        else:
            do_send_fn = self.do_send_image

        if result_request_upload_iq_protocol_entity.isDuplicate():
            do_send_fn(path, result_request_upload_iq_protocol_entity.getUrl(), jid,
                       result_request_upload_iq_protocol_entity.getIp(), caption)
        else:
            success_fn = lambda filePath, jid, url: do_send_fn(filePath, url, jid, 
                                                               result_request_upload_iq_protocol_entity.getIp(), 
                                                               caption)
            mediaUploader = MediaUploader(jid, self.getOwnJid(), path,
                                          result_request_upload_iq_protocol_entity.getUrl(),
                                          result_request_upload_iq_protocol_entity.getResumeOffset(),
                                          success_fn, self.on_upload_error, self.on_upload_progress, async=True)
            mediaUploader.start()
            
    def on_upload_error(self, file_path, jid, url):
        logger.error("Upload file %s to %s for %s failed!" % (file_path, url, jid))

    def on_upload_progress(self, file_path, jid, url, progress):
        logger.info("%s => %s, %d%% \r" % (os.path.basename(file_path), jid, progress))
    
    @connection_required
    def send_message(self, number, content):
        """
        Send message
        :param str number: phone number with cc (country code)
        :param str content: body text of the message
        """
        outgoing_message = TextMessageProtocolEntity(content.encode("utf-8") if sys.version_info >= (3, 0)
                                                     else content, to=self.normalize_jid(number))
        self.toLower(outgoing_message)
        return outgoing_message
        
    def connect(self):
        if self.connected:
            logger.warning("Already connected, disconnect first")
            return False
        self.getLayerInterface(YowNetworkLayer).connect()
        return True

    @connection_required
    def disconnect(self):
        self.broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_DISCONNECT))
        return True
    
    def _send_media_path(self, number, path, type, caption=None):
        jid = self.normalize_jid(number)
        entity = RequestUploadIqProtocolEntity(type, filePath=path)
        success_fn = lambda success_entity, original_entity: self.on_request_upload_result(jid, path, success_entity, 
                                                                                           original_entity, caption)
        error_fn = lambda error_entity, original_entity: self.on_request_upload_error(jid, path, error_entity, 
                                                                                      original_entity)
        self._sendIq(entity, success_fn, error_fn)

    @connection_required
    def send_image(self, number, path, caption=None):
        """
        Send image message
        :param str number: phone number with cc (country code)
        :param str path: image file path
        """
        return self._send_media_path(number, path, RequestUploadIqProtocolEntity.MEDIA_TYPE_IMAGE, caption)
        
    @connection_required
    def send_audio(self, number, path):
        """
        Send audio message
        :param str number: phone number with cc (country code)
        :param str path: audio file path
        """
        return self._send_media_path(number, path, RequestUploadIqProtocolEntity.MEDIA_TYPE_AUDIO)
    
    @connection_required
    def send_location(self, number, name, url, latitude, longitude):
        """
        Send location message
        :param str number: phone number with cc (country code)
        :param str name: indentifier for the location
        :param str url: location url
        :param str longitude: location longitude
        :param str latitude: location latitude 
        """
        location_message = LocationMediaMessageProtocolEntity(latitude, longitude, name, url, encoding="raw", 
                                                              to=self.normalize_jid(number))
        self.toLower(location_message)
        return location_message
    
    @connection_required
    def send_vcard(self, number, name, data):
        """
        Send location message
        :param str number: phone number with cc (country code)
        :param str name: indentifier for the location
        :param str data: vcard format i.e.
        BEGIN:VCARD
        VERSION:3.0
        N:;Home;;;
        FN:Home
        item1.TEL:+34 911234567
        item1.X-ABLabel:Home
        END:VCARD
        """
        vcard_message = VCardMediaMessageProtocolEntity(name, data, to=self.normalize_jid(number))
        self.toLower(vcard_message)
        return vcard_message


from __future__ import absolute_import
from celery import bootsteps
from yowsup_celery.stack import YowsupStack
from yowsup_celery.utils import import_string
import logging
from yowsup_celery.exceptions import ConfigurationError

logger = logging.getLogger(__name__)

class YowsupStep(bootsteps.StartStopStep):    
    
    def _get_top_layers(self, worker):
        top_layers_string = worker.app.conf.table().get('TOP_LAYERS', None)
        top_layers = []
        if top_layers_string:
            for top_layer_string in top_layers_string:
                top_layer = import_string(top_layer_string)
                top_layers.append(top_layer)
        return tuple(top_layers)    
    
    def _get_config(self, config):
        try:
            with open(config, 'r') as f:
                out = {}
                for l in f:
                    line = l.strip()
                    if len(line) and line[0] not in ('#', ';'):
                        prep = line.split('#', 1)[0].split(';', 1)[0].split('=', 1)
                        varname = prep[0].strip()
                        val = prep[1].strip()
                        out[varname.replace('-', '_')] = val
                return out
        except IOError:
            logger.error("Invalid config path: %s" % config)
            raise ConfigurationError("Invalid config path: %s" % config)
    
    def _get_credentials(self, login, config):
        if login:
            return tuple(login.split(":"))
        elif config:
            config_credentials = self._get_config(config)
            if "password" not in config_credentials or "phone" not in config_credentials:
                raise ConfigurationError("Must specify at least phone number and password in config file")
            return config_credentials["phone"], config_credentials["password"]
        else:
            return None
        
    def __init__(self, worker, login, config, unmoxie, **kwargs):
        """
        :param worker: celery worker
        :param login: optional login:password parameter
        :param config: optional path to configuration file
        :param unmoxie: boolean to disable encryption
        """
        credentials = self._get_credentials(login, config)
        if not credentials:
            raise ConfigurationError("Error: You must specify a configuration method")
        worker.app.stack = YowsupStack(credentials, not unmoxie, self._get_top_layers(worker))
        logger.info("Yowsup for %s intialized" % credentials[0])

    def stop(self, worker):     
        logger.info("Stopping yowsup")
        if worker.app.stack.facade.connected():
            worker.app.stack.facade.disconnect()
            logger.info("Disconnect yowsup")



# -*- coding: utf-8 -*-
class YowsupCeleryError(Exception):
    pass


class UnexpectedError(YowsupCeleryError):
    """ Raised for unknown or unexpected errors. """
    pass


class ConfigurationError(YowsupCeleryError):
    """
    Raised when YowsupStack detects and error in configurations
    """
    pass


class ConnectionError(YowsupCeleryError):
    """
    Raised when CeleryLayer tries to perform an action which requires to be
    connected to WhatsApp
    """
    pass


class AuthenticationError(YowsupCeleryError):
    """
    Raised when YowsupStack cannot authenticate with the whatsapp.  This means the
    password for number is incorrect. Check if registration was correct
    """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import inspect
import threading
from yowsup.layers.interface import YowInterfaceLayer
from yowsup.layers.network import YowNetworkLayer
from yowsup.layers import YowLayerEvent, EventCallback
from tests.utils import success_protocol_entity, failure_protocol_entity, ack_incoming_protocol_entity, \
    text_message_protocol_entity, receipt_incoming_protocol_entity
from yowsup.layers.auth.autherror import AuthError
from yowsup_celery.stack import YowsupStack
from yowsup_celery.layer import CeleryLayer


try:
    from unittest import mock
except ImportError:
    import mock  # noqa
    

class SendProtocolEntityCallback(object):
    """
    Decorate callback for handling send protoocol entities
    """
    def __init__(self, entityType):
        self.entityType = entityType

    def __call__(self, fn):
        fn.send_entity_callback = self.entityType
        return fn

class CoreLayerMock(YowInterfaceLayer):
    """
    Mock layer to simulate core layers
    """
    
    def __init__(self, error_auth=False, error_ack=False):
        super(CoreLayerMock, self).__init__()
        self.send_entity_callbacks = {}
        self.error_auth = error_auth
        self.error_ack = error_ack
        self.connected = False
        self.lowerSink = []
        self.toLower = self.sendOverrider
        members = inspect.getmembers(self, predicate=inspect.ismethod)
        for m in members:
            if hasattr(m[1], "send_entity_callback"):
                fname = m[0]
                fn = m[1]
                self.send_entity_callbacks[fn.send_entity_callback] = getattr(self, fname)
                
    def sendOverrider(self, data):
        self.lowerSink.append(data)
                
    def send(self, entity):
        if not self.processIqRegistry(entity):
            entityType = entity.getTag()
            if entityType in self.send_entity_callbacks:
                self.send_entity_callbacks[entityType](entity)
            else:
                self.toLower(entity)

    @EventCallback(YowNetworkLayer.EVENT_STATE_CONNECT)
    def on_connect(self, event):
        if self.error_auth:
            self.connected = False
            protocol_entity = failure_protocol_entity()
        else:
            self.connected = True
            protocol_entity = success_protocol_entity()
        self.toUpper(protocol_entity)
        if not self.connected:
            raise AuthError()

    @EventCallback(YowNetworkLayer.EVENT_STATE_DISCONNECT)
    def on_disconnect(self, event):
        self.connected = False
        self.emitEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_DISCONNECTED, 
                                     reason="Connection closed", 
                                     detached=True))
        return True
    
    @SendProtocolEntityCallback("message")
    def on_message(self, message_protocol_entity):
        if not self.error_ack:
            self.toUpper(ack_incoming_protocol_entity(message_protocol_entity))
        self.toLower(message_protocol_entity)
        
    @SendProtocolEntityCallback("ack")
    def on_ack(self, ack_protocol_entity):
        self.toLower(ack_protocol_entity)
    
    @SendProtocolEntityCallback("entity")
    def on_receipt(self, receipt_protocol_entity):
        self.toLower(receipt_protocol_entity)
    
    def connect(self):
        if self.error_auth:
            self.connected = False
            protocol_entity = failure_protocol_entity()
        else:
            self.connected = True
            protocol_entity = success_protocol_entity()
        self.toUpper(protocol_entity)
        if not self.connected:
            raise AuthError()
        
    def receive_message(self):
        msg = text_message_protocol_entity()
        self.toUpper(msg)
        return msg
        
    def receive_ack(self):
        ack = ack_incoming_protocol_entity()
        self.toUpper(ack)
        return ack
        
    def receive_receipt(self):
        receipt = receipt_incoming_protocol_entity()
        self.toUpper(receipt)
        return receipt
        
    
class StackTest(unittest.TestCase):
    
    def setUp(self):
        self.stack = YowsupStack(("341111111", "password"))
        stackClassesArr = (CeleryLayer, CoreLayerMock)   
        self.stack._YowStack__stack = stackClassesArr[::-1]
        self.stack._YowStack__stackInstances = []
        self.stack._YowStack__props = {}
        self.stack._construct()
        self.stack.facade = self.stack.getLayerInterface(CeleryLayer)
        self.mock_layer = self.stack._YowStack__stackInstances[0]
        self.celery_layer = self.stack._YowStack__stackInstances[1]
        self.number = "341234567"
        self.content = "message test"
        self.message = (self.number, self.content)
        
    def _queue_thread(self, fn, *args, **kwargs):
        while not self.stack.listening:
            pass
        if fn:
            fn(*args, **kwargs)
        
    def _asynloop(self, asyn_action, timeout=0.1, auto_connect=False):
        self.input_thread = threading.Thread(target=self._queue_thread, args=(asyn_action,))
        self.input_thread.daemon = True
        self.input_thread.start()
        self.stack.asynloop(timeout=timeout, auto_connect=auto_connect)
        self.assertFalse(self.stack.listening, False)
        self.assertTrue(self.stack.detached_queue.empty())
        
    def test_asynloop_timeout(self):
        self._asynloop(None, 0.1)     
        
    def test_asynloop_autoconnect(self):
        def check_connected():
            self.assertTrue(self.stack.facade.connected())
        self._asynloop(check_connected, auto_connect=True) 
        
    def _test_asynloop(self, action):
        def queue_to_loop():
            self.assertTrue(self.stack.listening)
            self.stack.facade.connect()
            self.assertTrue(self.stack.facade.connected())
            if action:
                action()
            self.stack.facade.disconnect()
            self.assertFalse(self.stack.facade.connected())
        
        with mock.patch.object(YowsupStack, 'getLayerInterface', return_value=self.mock_layer) as mock_interface:
            mock_interface.return_value = self.mock_layer
            self._asynloop(queue_to_loop)
            
    def test_asynloop_disconnect(self):
        self._test_asynloop(None)
        
    def test_asynloop_receive_message(self):
        def receive_message():
            msg = self.mock_layer.receive_message()
            ack = self.mock_layer.lowerSink.pop()
            self.assertEqual(msg.getId(), ack.getId())
        self._test_asynloop(receive_message)
        
    def test_asynloop_receive_ack(self):
        def receive_ack():
            self.mock_layer.receive_ack()
            self.assertEqual([], self.mock_layer.lowerSink)         
        self._test_asynloop(receive_ack)
        
    def test_asynloop_receive_receipt(self):
        def receive_receipt():
            receipt = self.mock_layer.receive_receipt()
            ack = self.mock_layer.lowerSink.pop()
            self.assertEqual(receipt.getId(), ack.getId())
        self._test_asynloop(receive_receipt)
        
    def test_asynloop_send_message(self):
        def send_message():
            msg_to_send = self.stack.facade.send_message("341234567", "content")
            msg_sent = self.mock_layer.lowerSink.pop()
            self.assertEqual(msg_to_send.getId(), msg_sent.getId())
            self.assertEqual(msg_to_send.getBody(), msg_sent.getBody())
        self._test_asynloop(send_message)



from __future__ import absolute_import

from celery import Task, shared_task
from functools import wraps

def listening_required(f):
    @wraps(f)
    def decorated_function(self, *args, **kwargs):
        if not self.stack.listening:
            listen.apply_async(queue=self.request.delivery_info['routing_key'])
            return self.retry()
        else:
            return f(self, *args, **kwargs)
    return decorated_function

   
class YowsupTask(Task):
    abstract = True
    default_retry_delay = 0.5
    
    @property
    def stack(self):
        return self.app.stack
    
    @property
    def facade(self):
        return self.app.stack.facade
    
    
@shared_task(base=YowsupTask, bind=True, ignore_result=True)
def listen(self):
    if not self.stack.listening:
        return self.stack.asynloop()
    else:
        return "Already listening"

@shared_task(base=YowsupTask, bind=True)
@listening_required
def connect(self):
    return self.facade.connect()
        

@shared_task(base=YowsupTask, bind=True)
@listening_required
def disconnect(self):
    return self.facade.disconnect()

@shared_task(base=YowsupTask, bind=True)
@listening_required
def send_message(self, number, content):
    self.facade.send_message(number, content)
    return True

@shared_task(base=YowsupTask, bind=True)
@listening_required
def send_image(self, number, path):
    self.facade.send_image(number, path)
    return True

@shared_task(base=YowsupTask, bind=True)
@listening_required
def send_audio(self, number, path):
    self.facade.send_audio(number, path)
    return True

@shared_task(base=YowsupTask, bind=True)
@listening_required
def send_location(self, number, name, url, latitude, longitude):
    self.facade.send_location(number, name, url, latitude, longitude)
    return True

@shared_task(base=YowsupTask, bind=True)
@listening_required
def send_vcard(self, number, name, data):
    self.facade.send_vcard(number, name, data)
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from yowsup.layers import YowProtocolLayerTest
from yowsup_celery.layer import CeleryLayer
from yowsup.layers.protocol_messages.protocolentities import TextMessageProtocolEntity
from yowsup.layers.protocol_receipts.protocolentities import IncomingReceiptProtocolEntity
import time
from yowsup_celery.exceptions import ConnectionError
from tests.utils import success_protocol_entity, failure_protocol_entity, \
    image_downloadable_media_message_protocol_entity, audio_downloadable_media_message_protocol_entity
from yowsup.layers import YowLayerEvent
from yowsup.layers.network import YowNetworkLayer
import unittest
from yowsup.layers.protocol_media.protocolentities.message_media_downloadable import \
    DownloadableMediaMessageProtocolEntity
try:
    from unittest import mock
except ImportError:
    import mock  # noqa
    
def init_request_upload(self, mediaType, filePath):
    self.tag = "iq"
    self._id = "id_iq"
    self._from = None
    self._type = "set"
    self.xmlns = "w:m"
    self.to = "s.whatsapp.net"
    self.mediaType = mediaType
    self.b64Hash = "hash"
    self.size = int("1234")
    self.origHash = "orighash"

class CeleryLayerTest(YowProtocolLayerTest, CeleryLayer):
    
    def setUp(self):
        CeleryLayer.__init__(self)
        self.connected = True
        self.number = "341234567"
        self.content = "content"
        
    def receive_message(self):
        content = "Received message"
        jid = "bbb@s.whatsapp.net"
        msg = TextMessageProtocolEntity(content, _from=jid)
        self.receive(msg)
        return msg
    
    def receive_receipt(self):
        receipt = IncomingReceiptProtocolEntity("123", "sender", int(time.time()))
        self.receive(receipt)
        return receipt    
    
    def test_already_connected(self):
        self.connected = True
        self.assertFalse(self.connect())
              
    def test_connection_successful(self):
        self.connected = False
        self.on_success(success_protocol_entity())
        self.assertTrue(self.connected)

    def test_connection_failure(self):
        self.connected = True
        self.on_failure(failure_protocol_entity())
        self.assertFalse(self.connected)
        
    def test_ack_for_received_message(self):
        msg = self.receive_message()
        ack = self.lowerSink.pop()
        self.assertEqual(ack.getId(), msg.getId())
        
    def test_ack_for_received_receipt(self):
        receipt = self.receive_receipt()
        ack = self.lowerSink.pop()
        self.assertEqual(ack.getId(), receipt.getId())
        
    def test_connection_required(self):
        self.connected = False
        self.assertRaises(ConnectionError, self.send_message, self.number, self.content)
        
    def test_send_message(self):
        msg = self.send_message(self.number, self.content)
        out_msg = self.lowerSink.pop()
        self.assertEqual(msg.getId(), out_msg.getId())
        self.assertEqual(self.number + '@s.whatsapp.net', out_msg.getTo())
        self.assertEqual(self.content, out_msg.getBody().decode('utf-8'))
        
    def test_disconnect(self):
        self.connected = True
        self.disconnect()
        evt = self.lowerEventSink.pop()
        self.assertEqual(evt.name, YowLayerEvent(YowNetworkLayer.EVENT_STATE_DISCONNECT).name)
        
    def test_connect(self):
        self.connected = False
        self.getLayerInterface = mock.MagicMock()
        self.getLayerInterface.return_value.connect = mock.MagicMock()
        self.connect()
        self.getLayerInterface.return_value.connect.assert_called_once_with()
         
    def test_on_disconnected(self):
        self.connected = True
        self.onEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_DISCONNECTED))
        self.assertFalse(self.connected)

    def _test_send_media_to_upload(self, fn, *args, **kwargs):
        with mock.patch('yowsup.layers.protocol_media.protocolentities.iq_requestupload.RequestUploadIqProtocolEntity.__init__',  # noqa
                        init_request_upload):  
            fn(*args, **kwargs)
            entity_iq = self.lowerSink.pop()
            iq_entity_registered, iq_on_success, iq_on_error = self.iqRegistry[entity_iq.getId()]
            self.assertEqual(entity_iq.getId(), iq_entity_registered.getId())
            self.assertEqual(entity_iq.getType(), iq_entity_registered.getType())            
    
    def test_send_image_to_upload(self):
        self._test_send_media_to_upload(self.send_image, "341234567", "path/image.jpg", "image caption")
                                       
    def test_send_audio_to_upload(self):
        self._test_send_media_to_upload(self.send_audio, "341234567", "path/audio.mp3")

    def test_send_image_to_do(self):
        path = "file_path/image.jpg"
        url = "image_url"
        to = "3412345670"
        ip = "192.12.1.1"
        caption = "caption"
        with mock.patch('yowsup.layers.protocol_media.protocolentities.message_media_downloadable_image.ImageDownloadableMediaMessageProtocolEntity.fromFilePath') as mock_image:  # noqa
            mock_image.return_value = image_downloadable_media_message_protocol_entity(url, to, ip, caption)
            self.do_send_image(path, url, to, ip, caption)
            entity = self.lowerSink.pop()
            self.assertEqual(entity.caption, caption)
            self.assertEqual(entity.to, to)
            self.assertEqual(entity.url, url)
            self.assertEqual(entity.getMediaType(), DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_IMAGE)
            
    def test_send_audio_to_do(self):
        path = "file_path/audio.mp3"
        url = "audio_url"
        to = "3412345670"
        ip = "192.12.1.1"     
        with mock.patch('yowsup.layers.protocol_media.protocolentities.message_media_downloadable_audio.AudioDownloadableMediaMessageProtocolEntity.fromFilePath') as mock_audio:  # noqa
            mock_audio.return_value = audio_downloadable_media_message_protocol_entity(url, to, ip)
            self.do_send_audio(path, url, to, ip)
            entity = self.lowerSink.pop()
            self.assertEqual(entity.to, to)
            self.assertEqual(entity.url, url)
            self.assertEqual(entity.getMediaType(), DownloadableMediaMessageProtocolEntity.MEDIA_TYPE_AUDIO)
            
    def test_send_location(self):
        params = {
            "number": "341234567",
            "name": "Cerro espino",
            "url": "https://foursquare.com/v/4ca5ffa797c8a1cd9edc6ba5",
            "latitude": "40.458197101060826",
            "longitude": "-3.8601064682006836"
        }
        location_message = self.send_location(**params)
        out_msg = self.lowerSink.pop()
        self.assertEqual(location_message.getId(), out_msg.getId())
        self.assertEqual(params["number"] + '@s.whatsapp.net', out_msg.getTo())
        self.assertEqual(params["name"], out_msg.getLocationName())
        self.assertEqual(params["url"], out_msg.getLocationURL())
        self.assertEqual(params["latitude"], out_msg.getLatitude())
        self.assertEqual(params["longitude"], out_msg.getLongitude())
        
    def test_send_vcard(self):
        params = {
            "number": "341234567", 
            "name": "Grandma",
            "data": """BEGIN:VCARD\nVERSION:3.0\nN:;Grandma;;;N:;Grandma;;;\nFN:Grandma\n
                item1.TEL:+34 1234567\nitem1.X-ABLabel:Home\nEND:VCARD\n"""
        }
        vcard_message = self.send_vcard(**params)
        out_msg = self.lowerSink.pop()
        self.assertEqual(vcard_message.getId(), out_msg.getId())
        self.assertEqual(params["number"] + '@s.whatsapp.net', out_msg.getTo())
        self.assertEqual(params["name"], out_msg.getName())
        self.assertEqual(params["data"], out_msg.getCardData())
        
        
if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



import asyncore
import time
import logging
import sys
import traceback
from yowsup import stacks
from yowsup_celery.layer import CeleryLayer
from yowsup_celery import exceptions
from yowsup.layers import YowLayerEvent
from yowsup.layers.auth import AuthError
from yowsup.layers.network import YowNetworkLayer
try:
    import Queue
except ImportError:
    import queue as Queue
    
logger = logging.getLogger(__name__)

class YowsupStack(stacks.YowStack):
    """
    Gateway for Yowsup in a client API way
    
    :ivar bool listening: asyncore loop task in execution
    :ivar Queue detached_queue: Queue with callbacks to execute after
    :ivar YowLayerInterface facade:layer interface on top of stack
    disconnection
    """
    
    def __init__(self, credentials, encryption=False, top_layers=None):
        """
        :param credentials: number and registed password
        :param bool encryptionEnabled:  E2E encryption enabled/ disabled
        :params top_layers: tuple of layer between :class:`yowsup_gateway.layer.CeleryLayer` 
        and Yowsup Core Layers  
        """
        top_layers = top_layers + (CeleryLayer,) if top_layers else (CeleryLayer,)
        layers = stacks.YowStackBuilder.getDefaultLayers(axolotl=encryption) + top_layers
        try:
            super(YowsupStack, self).__init__(layers, reversed=False)
        except ValueError as e:
            raise exceptions.ConfigurationError(e.args[0])
        self.setCredentials(credentials)
        self.detached_queue = Queue.Queue()
        self.facade = self.getLayerInterface(CeleryLayer)
        self.listening = False
        
    def execDetached(self, fn):
        return self.detached_queue.put(fn)
            
    def cleanup(self):
        self.listening = False
        self.detached_queue = Queue.Queue()

    def asynloop(self, auto_connect=False, timeout=10, detached_delay=0.2):
        """
        Non-blocking event loop consuming messages until connection is lost,
           or shutdown is requested.
        :param int timeout: number of secs for asyncore timeout
        :param float detached_delay: float secs to sleep when exiting asyncore loop and execution detached queue
            callbacks
        """
        if auto_connect:
            self.broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_CONNECT))
        try:
            self.listening = True
            start = int(time.time())
            while True:
                asyncore.loop(timeout)
                time.sleep(detached_delay)
                try:
                    # Execute from detached queue callback
                    callback = self.detached_queue.get(False)
                    callback()
                except Queue.Empty:
                    pass
                if int(time.time()) - start > timeout:
                    logger.info("Asynloop : Timeout")
                    #  defensive code should be already disconneted
                    if self.facade.connected():
                        self.broadcastEvent(YowLayerEvent(YowNetworkLayer.EVENT_STATE_DISCONNECT))
                    break
            self.cleanup()
        except AuthError as e:
            self.cleanup()
            raise exceptions.AuthenticationError("Authentication Error: {0}".format(e))
        except:
            self.cleanup()
            exc_info = sys.exc_info()
            traceback.print_exception(*exc_info)
            raise exceptions.UnexpectedError(str(exc_info[0]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pip

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
                'celery>=3.1.19',
                'gevent>=1.0.2',
                #install fork git+https://github.com/jlmadurga/yowsup.git@issue_1181#egg=yowsup 
                'yowsup2'
                ]
test_requirements = requirements + [
                     'mock>=1.3.0'
                     ]


setup(
    name='yowsup-celery',
    version='0.1.5',
    description="Yowsup integrated in a celery architecture",
    long_description=readme + '\n\n' + history,
    author="Juan Madurga",
    author_email='jlmadurga@gmail.com',
    url='https://github.com/jlmadurga/yowsup-celery',
    packages=[
        'yowsup_celery',
    ],
    package_dir={'yowsup_celery':
                 'yowsup_celery'},
    include_package_data=True,
    install_requires= requirements,
    license="ISCL",
    zip_safe=False,
    keywords='yowsup-celery',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
from yowsup_celery.steps import YowsupStep
from yowsup_celery.stack import YowsupStack
from yowsup_celery.layer_interface import CeleryLayerInterface
from yowsup_celery.exceptions import ConfigurationError
try:
    from unittest import mock
except ImportError:
    import mock  # noqa

class TestYowsupStep(unittest.TestCase):

    def setUp(self):
        self.worker = mock.Mock()
        self.worker.app.conf.table = mock.MagicMock(return_value={})

    def tearDown(self):
        pass
    
    def assertStackIntiliazed(self, stack):
        self.assertIsInstance(stack, YowsupStack)
        self.assertFalse(stack.listening)
        self.assertIsInstance(stack.facade, CeleryLayerInterface)
        
    def _correct_login_step(self):
        return YowsupStep(self.worker, "341234567:password", None, True)

    def test_init_login_ok(self):
        YowsupStep(self.worker, "341234567:password", None, True)
        self.assertStackIntiliazed(self.worker.app.stack)
        
    def test_init_conf_ok(self):
        with mock.patch('six.moves.builtins.open', mock.mock_open(read_data='adasdasd')) as m:
            handle = m.return_value.__enter__.return_value
            handle.__iter__.return_value = iter(["####confifle\n", "phone=341111111\n", "password=asdasdasdasd\n"])
            YowsupStep(self.worker, None, "file_path", True)
            self.assertStackIntiliazed(self.worker.app.stack)
    
    def test_init_conf_file_not_complete(self):
        with mock.patch('six.moves.builtins.open', mock.mock_open(read_data='adasdasd')) as m:
            handle = m.return_value.__enter__.return_value
            handle.__iter__.return_value = iter(["####confifle\n", "password=asdasdasdasd\n"])
            self.assertRaises(ConfigurationError, YowsupStep, self.worker, None, "file_path", True)
    
    def test_init_conf_file_not_exists(self):
        self.assertRaises(ConfigurationError, YowsupStep, self.worker, None, "file_path", True)        
        
    def test_init_no_configuration_method(self):
        self.assertRaises(ConfigurationError, YowsupStep, self.worker, None, None, True)
        
    def test_init_top_layer_not_interface_layer(self):
        self.worker.app.conf.table = mock.MagicMock(return_value={'TOP_LAYERS': ('yowsup.stacks.yowstack.YowStack',)})
        self.assertRaises(ConfigurationError, YowsupStep, self.worker, "341234567:password", None, True)
        
    def test_init_top_layer_not_module(self):
        self.worker.app.conf.table = mock.MagicMock(return_value={'TOP_LAYERS': ('not_module',)})
        self.assertRaises(ImportError, YowsupStep, self.worker, "341234567:password", None, True)
        
    def test_init_top_layer_not_class(self):
        self.worker.app.conf.table = mock.MagicMock(return_value={'TOP_LAYERS': ('yowsup.stacks.yowstack.NoDefined',)})
        self.assertRaises(ImportError, YowsupStep, self.worker, "341234567:password", None, True)
        
    def test_stop_when_connected(self):
        step = self._correct_login_step()
        self.worker.app.stack.facade.connected = mock.MagicMock(return_value=True)
        self.worker.app.stack.facade.disconnect = mock.MagicMock()
        step.stop(self.worker)
        self.worker.app.stack.facade.disconnect.assert_called_once_with()
        
    def test_stop_not_connected(self):
        step = self._correct_login_step()
        self.worker.app.stack.facade.connected = mock.MagicMock(return_value=False)
        self.worker.app.stack.facade.disconnect = mock.MagicMock()
        step.stop(self.worker)
        self.assertEqual(0, self.worker.app.stack.facade.disconnect.call_count)

if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



from __future__ import absolute_import

from celery import Celery
from yowsup_celery.steps import YowsupStep
from celery.bin import Option

class YowCelery(Celery):
    
    def __init__(self, *args, **kwargs):
        super(YowCelery, self).__init__(*args, **kwargs)
        self.steps['worker'].add(YowsupStep)
        self.user_options['worker'].update(
            [Option('--yowlogin', dest='login', metavar="phone:b64password", default=None, 
                    help='''WhatsApp login credentials, in the format phonenumber:password, 
                    where password is base64 encoded.'''),
             Option('--yowconfig', dest='config', default=None, 
                    help='Path to config file containing authentication info.'),
             Option('--yowunmoxie', dest='unmoxie', action="store_true", default=False, 
                    help="Disable E2E Encryption")])



from yowsup.layers import YowLayerInterface


class CeleryLayerInterface(YowLayerInterface):
    
    def connect(self):
        return self._layer.connect()
    
    def disconnect(self):
        return self._layer.disconnect()
    
    def send_message(self, number, content):
        return self._layer.send_message(number, content)
    
    def send_image(self, number, path):
        return self._layer.send_image(number, path)
    
    def send_audio(self, number, path):
        return self._layer.send_audio(number, path)
    
    def send_location(self, number, name, url, latitude, longitude):
        return self._layer.send_location(number, name, url, latitude, longitude)
    
    def send_vcard(self, number, name, data):
        return self._layer.send_vcard(number, name, data)

    def connected(self):
        return self._layer.connected
